@@ -147,7 +147,9 @@ impl BatteryStats {
         // charge_now / charge_full
         let soc = match (stats.charge_now, stats.charge_full) {
             (Some(c_n), Some(c_f)) => Some(SoC::new(c_n as f32 / c_f as f32)?),
-            _ => None,
+            _ => stats
+                .capacity
+                .and_then(|val| SoC::new(val as f32 / 100.0).ok()),
         };
         // charge_full / charge_full_design
         let health = match (stats.charge_full, stats.charge_full_design) {
@@ -246,13 +248,18 @@ impl Battery {
         for entry in std::fs::read_dir(&sysfs)? {
             let entry = entry?;
             let entry_name = entry.file_name().to_str().unwrap().to_string();
-            debug!("Checking file {}", entry.path().display());
-            debug!("File name: {}", entry_name);
 
-            if entry_name.starts_with("BAT")
-                || entry_name.starts_with("CMB")
-                || entry_name.starts_with("battery")
-            {
+            debug!("Checking sysfs entry {}", entry.path().display());
+
+            if !std::fs::metadata(entry.path())?.file_type().is_dir() {
+                debug!("Skipping non-directory {}", entry_name);
+                continue;
+            }
+
+            let pwr_type = std::fs::read_to_string(entry.path().join("type"))?;
+            debug!("Has type {}", pwr_type);
+
+            if pwr_type.starts_with("Battery") {
                 return Battery::new(&entry.path());
             }
         }
@@ -264,16 +271,37 @@ impl Battery {
 impl std::fmt::Display for Battery {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let stats = self.stats().unwrap();
+        let value_unknown = String::from("?");
+
+        let v = match stats.voltage {
+            Some(value) => format!("{}", Unit::V(value)),
+            None => value_unknown.clone(),
+        };
+        let a = match stats.current {
+            Some(value) => format!("{}", Unit::A(value)),
+            None => value_unknown.clone(),
+        };
+        let w = match stats.power {
+            Some(value) => format!("{}", Unit::W(value)),
+            None => value_unknown.clone(),
+        };
+        let soc = match stats.soc {
+            Some(value) => format!("{}", value),
+            None => value_unknown.clone(),
+        };
+        let health_text = match stats.health.as_ref() {
+            Some(value) => value.text(),
+            None => value_unknown.clone(),
+        };
+        let health = match stats.health {
+            Some(value) => format!("{}", value),
+            None => value_unknown,
+        };
+
         write!(
             f,
-            "Battery: {}\n\n- Voltage : {}\n- Current : {}\n- Power   : {}\n- Capacity: {}\n- Health  : {} ({:.2})\n",
-            self.path.display(),
-            Unit::V(stats.voltage.unwrap()),
-            Unit::A(stats.current.unwrap()),
-            Unit::W(stats.power.unwrap()),
-            stats.soc.unwrap(),
-            stats.health.as_ref().unwrap().text(),
-            stats.health.unwrap(),
+            "Battery: {}\n\n- Voltage : {}\n- Current : {}\n- Power   : {}\n- Capacity: {}\n- Health  : {} ({})\n",
+            self.path.display(), v, a, w, soc, health_text, health
         )
     }
 }
